@@ -32,7 +32,7 @@
 #include "file.h"
 
 #ifndef	lint
-FILE_RCSID("@(#)$File: file.c,v 1.187 2020/06/07 17:38:30 christos Exp $")
+FILE_RCSID("@(#)$File: file.c,v 1.189 2021/02/05 21:33:49 christos Exp $")
 #endif	/* lint */
 
 #include "magic.h"
@@ -146,6 +146,8 @@ private struct {
 	    "max ELF prog sections processed" },
 	{ "elf_shnum",	MAGIC_PARAM_ELF_SHNUM_MAX, 0, 0, FILE_ELF_SHNUM_MAX,
 	    "max ELF sections processed" },
+	{ "encoding",	MAGIC_PARAM_ENCODING_MAX, 0, 0, FILE_ENCODING_MAX,
+	    "max bytes to scan for encoding" },
 	{ "indir",	MAGIC_PARAM_INDIR_MAX, 0, 0, FILE_INDIR_MAX,
 	    "recursion limit for indirection" },
 	{ "name",	MAGIC_PARAM_NAME_MAX, 0, 0, FILE_NAME_MAX,
@@ -523,8 +525,6 @@ unwrap(struct magic_set *ms, const char *fn)
 		if (line[len - 1] == '\n')
 			line[len - 1] = '\0';
 		e |= process(ms, line, wid);
-		if(nobuffer)
-			(void)fflush(stdout);
 	}
 
 	free(line);
@@ -556,11 +556,12 @@ process(struct magic_set *ms, const char *inname, int wid)
 
 	if (type == NULL) {
 		(void)printf("ERROR: %s%c", magic_error(ms), c);
-		return 1;
 	} else {
 		(void)printf("%s%c", type, c);
-		return 0;
 	}
+	if (nobuffer)
+		(void)fflush(stdout);
+	return type == NULL;
 }
 
 protected size_t
