@@ -27,7 +27,7 @@
 #include "file.h"
 
 #ifndef lint
-FILE_RCSID("@(#)$File: readelf.c,v 1.173 2020/06/07 22:12:54 christos Exp $")
+FILE_RCSID("@(#)$File: readelf.c,v 1.175 2020/12/17 20:43:37 christos Exp $")
 #endif
 
 #ifdef BUILTIN_ELF
@@ -354,7 +354,7 @@ dophn_core(struct magic_set *ms, int clazz, int swap, int fd, off_t off,
 	size_t offset, len;
 	unsigned char nbuf[BUFSIZ];
 	ssize_t bufsize;
-	off_t ph_off = off;
+	off_t ph_off = off, offs;
 	int ph_num = num;
 
 	if (ms->flags & MAGIC_MIME)
@@ -377,8 +377,11 @@ dophn_core(struct magic_set *ms, int clazz, int swap, int fd, off_t off,
 	for ( ; num; num--) {
 		if (pread(fd, xph_addr, xph_sizeof, off) <
 		    CAST(ssize_t, xph_sizeof)) {
-			file_badread(ms);
-			return -1;
+			if (file_printf(ms, 
+			    ", can't read elf program headers at %jd",
+			    (intmax_t)off) == -1)
+				return -1;
+			return 0;
 		}
 		off += size;
 
@@ -395,9 +398,12 @@ dophn_core(struct magic_set *ms, int clazz, int swap, int fd, off_t off,
 		 * in the section.
 		 */
 		len = xph_filesz < sizeof(nbuf) ? xph_filesz : sizeof(nbuf);
-		if ((bufsize = pread(fd, nbuf, len, xph_offset)) == -1) {
-			file_badread(ms);
-			return -1;
+		offs = xph_offset;
+		if ((bufsize = pread(fd, nbuf, len, offs)) == -1) {
+			if (file_printf(ms, " can't read note section at %jd",
+			    (intmax_t)offs) == -1)
+				return -1;
+			return 0;
 		}
 		offset = 0;
 		for (;;) {
@@ -941,8 +947,12 @@ get_offset_from_virtaddr(struct magic_set *ms, int swap, int clazz, int fd,
 	for ( ; num; num--) {
 		if (pread(fd, xph_addr, xph_sizeof, off) <
 		    CAST(ssize_t, xph_sizeof)) {
-			file_badread(ms);
-			return -1;
+			if (file_printf(ms,
+			    ", can't read elf program header at %jd",
+			    (intmax_t)off) == -1)
+				return -1;
+			return 0;
+
 		}
 		off += xph_sizeof;
 
@@ -972,7 +982,9 @@ get_string_on_virtaddr(struct magic_set *ms,
 	    fsize, virtaddr);
 	if (offset < 0 ||
 	    (buflen = pread(fd, buf, CAST(size_t, buflen), offset)) <= 0) {
-		file_badread(ms);
+		if (file_printf(ms, ", can't read elf string at %jd",
+		    (intmax_t)offset) == -1)
+			return -1;
 		return 0;
 	}
 
@@ -1099,7 +1111,7 @@ do_auxv_note(struct magic_set *ms, unsigned char *nbuf, uint32_t type,
 
 private size_t
 dodynamic(struct magic_set *ms, void *vbuf, size_t offset, size_t size,
-    int clazz, int swap)
+    int clazz, int swap, int *pie, size_t *need)
 {
 	Elf32_Dyn dh32;
 	Elf64_Dyn dh64;
@@ -1117,11 +1129,15 @@ dodynamic(struct magic_set *ms, void *vbuf, size_t offset, size_t size,
 
 	switch (xdh_tag) {
 	case DT_FLAGS_1:
+		*pie = 1;
 		if (xdh_val & DF_1_PIE)
 			ms->mode |= 0111;
 		else
 			ms->mode &= ~0111;
 		break;
+	case DT_NEEDED:
+		(*need)++;
+		break;
 	default:
 		break;
 	}
@@ -1334,7 +1350,7 @@ doshn(struct magic_set *ms, int clazz, int swap, int fd, off_t off, int num,
 	int stripped = 1, has_debug_info = 0;
 	size_t nbadcap = 0;
 	void *nbuf;
-	off_t noff, coff, name_off;
+	off_t noff, coff, name_off, offs;
 	uint64_t cap_hw1 = 0;	/* SunOS 5.x hardware capabilities */
 	uint64_t cap_sf1 = 0;	/* SunOS 5.x software capabilities */
 	char name[50];
@@ -1355,9 +1371,10 @@ doshn(struct magic_set *ms, int clazz, int swap, int fd, off_t off, int num,
 	}
 
 	/* Read offset of name section to be able to read section names later */
-	if (pread(fd, xsh_addr, xsh_sizeof, CAST(off_t, (off + size * strtab)))
-	    < CAST(ssize_t, xsh_sizeof)) {
-		if (file_printf(ms, ", missing section headers") == -1)
+	offs = CAST(off_t, (off + size * strtab));
+	if (pread(fd, xsh_addr, xsh_sizeof, offs) < CAST(ssize_t, xsh_sizeof)) {
+		if (file_printf(ms, ", missing section headers at %jd",
+		    (intmax_t)offs) == -1)
 			return -1;
 		return 0;
 	}
@@ -1372,10 +1389,14 @@ doshn(struct magic_set *ms, int clazz, int swap, int fd, off_t off, int num,
 
 	for ( ; num; num--) {
 		/* Read the name of this section. */
-		if ((namesize = pread(fd, name, sizeof(name) - 1,
-		    name_off + xsh_name)) == -1) {
-			file_badread(ms);
-			return -1;
+		offs = name_off + xsh_name;
+		if ((namesize = pread(fd, name, sizeof(name) - 1, offs))
+		    == -1) {
+			if (file_printf(ms, 
+			    ", can't read name of elf section at %jd",
+			    (intmax_t)offs) == -1)
+				return -1;
+			return 0;
 		}
 		name[namesize] = '\0';
 		if (strcmp(name, ".debug_info") == 0) {
@@ -1385,8 +1406,10 @@ doshn(struct magic_set *ms, int clazz, int swap, int fd, off_t off, int num,
 
 		if (pread(fd, xsh_addr, xsh_sizeof, off) <
 		    CAST(ssize_t, xsh_sizeof)) {
-			file_badread(ms);
-			return -1;
+			if (file_printf(ms, ", can't read elf section at %jd",
+			    (intmax_t)off) == -1)
+				return -1;
+			return 0;
 		}
 		off += size;
 
@@ -1427,11 +1450,15 @@ doshn(struct magic_set *ms, int clazz, int swap, int fd, off_t off, int num,
 				    " for note");
 				return -1;
 			}
-			if (pread(fd, nbuf, xsh_size, xsh_offset) <
+			offs = xsh_offset;
+			if (pread(fd, nbuf, xsh_size, offs) <
 			    CAST(ssize_t, xsh_size)) {
-				file_badread(ms);
 				free(nbuf);
-				return -1;
+				if (file_printf(ms,
+				    ", can't read elf note at %jd",
+				    (intmax_t)offs) == -1)
+					return -1;
+				return 0;
 			}
 
 			noff = 0;
@@ -1608,9 +1635,10 @@ doshn(struct magic_set *ms, int clazz, int swap, int fd, off_t off, int num,
 }
 
 /*
- * Look through the program headers of an executable image, searching
- * for a PT_INTERP section; if one is found, it's dynamically linked,
- * otherwise it's statically linked.
+ * Look through the program headers of an executable image, to determine
+ * if it is statically or dynamically linked. If it has a dynamic section,
+ * it is pie, and does not have an interpreter or needed libraries, we
+ * call it static pie.
  */
 private int
 dophn_exec(struct magic_set *ms, int clazz, int swap, int fd, off_t off,
@@ -1619,12 +1647,13 @@ dophn_exec(struct magic_set *ms, int clazz, int swap, int fd, off_t off,
 {
 	Elf32_Phdr ph32;
 	Elf64_Phdr ph64;
-	const char *linking_style = "statically";
+	const char *linking_style;
 	unsigned char nbuf[BUFSIZ];
 	char ibuf[BUFSIZ];
 	char interp[BUFSIZ];
 	ssize_t bufsize;
-	size_t offset, align, len;
+	size_t offset, align, len, need = 0;
+	int pie = 0, dynamic = 0;
 
 	if (num == 0) {
 		if (file_printf(ms, ", no program header") == -1)
@@ -1642,8 +1671,11 @@ dophn_exec(struct magic_set *ms, int clazz, int swap, int fd, off_t off,
 		int doread;
 		if (pread(fd, xph_addr, xph_sizeof, off) <
 		    CAST(ssize_t, xph_sizeof)) {
-			file_badread(ms);
-			return -1;
+			if (file_printf(ms,
+			    ", can't read elf program headers at %jd",
+			    (intmax_t)off) == -1)
+				return -1;
+			return 0;
 		}
 
 		off += size;
@@ -1654,7 +1686,6 @@ dophn_exec(struct magic_set *ms, int clazz, int swap, int fd, off_t off,
 		switch (xph_type) {
 		case PT_DYNAMIC:
 			doread = 1;
-			linking_style = "dynamically";
 			break;
 		case PT_NOTE:
 			if (sh_num)	/* Did this through section headers */
@@ -1683,10 +1714,14 @@ dophn_exec(struct magic_set *ms, int clazz, int swap, int fd, off_t off,
 		if (doread) {
 			len = xph_filesz < sizeof(nbuf) ? xph_filesz
 			    : sizeof(nbuf);
-			bufsize = pread(fd, nbuf, len, xph_offset);
+			off_t offs = xph_offset;
+			bufsize = pread(fd, nbuf, len, offs);
 			if (bufsize == -1) {
-				file_badread(ms);
-				return -1;
+				if (file_printf(ms,
+				    ", can't read section at %jd",
+				    (intmax_t)offs) == -1)
+					return -1;
+				return 0;
 			}
 		} else
 			len = 0;
@@ -1694,6 +1729,7 @@ dophn_exec(struct magic_set *ms, int clazz, int swap, int fd, off_t off,
 		/* Things we can determine when we seek */
 		switch (xph_type) {
 		case PT_DYNAMIC:
+			dynamic = 1;
 			offset = 0;
 			// Let DF_1 determine if we are PIE or not.
 			ms->mode &= ~0111;
@@ -1701,7 +1737,8 @@ dophn_exec(struct magic_set *ms, int clazz, int swap, int fd, off_t off,
 				if (offset >= CAST(size_t, bufsize))
 					break;
 				offset = dodynamic(ms, nbuf, offset,
-				    CAST(size_t, bufsize), clazz, swap);
+				    CAST(size_t, bufsize), clazz, swap,
+				    &pie, &need);
 				if (offset == 0)
 					break;
 			}
@@ -1710,6 +1747,7 @@ dophn_exec(struct magic_set *ms, int clazz, int swap, int fd, off_t off,
 			break;
 
 		case PT_INTERP:
+			need++;
 			if (ms->flags & MAGIC_MIME)
 				continue;
 			if (bufsize && nbuf[0]) {
@@ -1744,8 +1782,15 @@ dophn_exec(struct magic_set *ms, int clazz, int swap, int fd, off_t off,
 	}
 	if (ms->flags & MAGIC_MIME)
 		return 0;
-	if (file_printf(ms, ", %s linked", linking_style)
-	    == -1)
+	if (dynamic) {
+		if (pie && need == 0)
+			linking_style = "static-pie";
+		else
+			linking_style = "dynamically";
+	} else {
+		linking_style = "statically";
+	}
+	if (file_printf(ms, ", %s linked", linking_style) == -1)
 		return -1;
 	if (interp[0])
 		if (file_printf(ms, ", interpreter %s",
